%% Load the Dinosaur dataset

W = load('rotdino.mat');
W = full(W.M_inliers');
W(W==0) = nan;

%% Initialize by filling in with mean values and taking SVD

W_init = W;
T = nanmean(W_init,1); %mean values
T = repmat(T, [size(W_init,1),1]);
W_init(isnan(W_init)) = T(isnan(W_init)); %replace nan's with mean values

% Take SVD, after first removing the translation vector
t = mean(W_init,1)'; 
W_init = W_init - repmat(t', [size(W_init,1) 1]);
[U,S,R] = svds(W_init,3);
R = R*S;
U = [U ones(size(U,1),1)];
R = [R t];




%% Run the algorithm


rng(1); %set random number generator for reproducable resuts

residual = U*R'-W;
error = sqrt(nanmean(residual(:).^2));
time = 0;

for iter = 1:1000
    % Run one iteration of the algorithm and time it
    timer = tic();
    [U,R] = grouse_iter(U,R,W,iter,100); %Use C=100 in the scaling
    timer = toc(timer);
    
    % Computer the RMSE
    residual = U*R'-W;
    rmse = sqrt(nanmean(residual(:).^2));
    
    % Update error and time measurements
    error = [error; rmse];
    time = [time; time(end)+timer];
    
    fprintf('Iteration %d, RMSE: %f\n', iter, error(end));
end


%% Estimate the 3D model

Structure = U';
Motion = R;

% Solve for Q
A = zeros(4*size(Motion,1)/2,9);
b = zeros(4*size(Motion,1)/2,1);
for jj = 1:size(Motion,1)/2
    x = Motion(2*(jj-1)+1,1:3)';
    y = Motion(2*jj,1:3)';
    
    % Add in typical rotation matrix constraints
    xx = x*x'; xx = xx(:)';
    xy = x*y'; xy = xy(:)';
    yx = y*x'; yx = yx(:)';
    yy = y*y'; yy = yy(:)';
    
    A(4*(jj-1)+1,:) =  xx-yy;
    b(4*(jj-1)+1) = 0;
    A(4*(jj-1)+2,:) =  yy-xx;
    b(4*(jj-1)+2) = 0;
    
    A(4*(jj-1)+3,:) = yx;
    b(4*(jj-1)+3) = 0;
    A(4*jj,:) =  xy;
    b(4*jj) = 0;
    
end
x = Motion(1,1:3)';
xx = x*x'; xx = xx(:)';
A = [A; xx];
b = [b; 1];

x = A\b;


QQT = reshape(x, [3 3]);
[vec, val] = eig(0.5*(QQT+QQT'));
vec = real(vec); val = real(val);
val = diag(val);
val = max(val, 1e-1);
Q = vec * diag(sqrt(val)) * vec';
Q = [Q zeros(3,1); zeros(1,3) 1];

Motion = Motion*Q;
Structure = Q\Structure;


model = Structure';
model = model(:,1:3);


%% Plot it
clf;

subplot(1,3,1);
loglog(time, error);
xlabel('Time (s)');
ylabel('2D RMSE');

subplot(1,3,2);
W_hat = U*R';
plot(W_hat(:,1:2:end)', W_hat(:,2:2:end)', 'k')
axis ij; axis equal;
title('Point tracks (should be near elliptical)');

subplot(1,3,3);
plot3(model(:,1), model(:,2), model(:,3), '.k');
axis equal;
title('Estimated 3D model');

